@@ -129,6 +129,7 @@ class BaseProxy(object):
         # __conn being created __del__() can detect the condition and handle it
         # correctly.
         self.__conn = None
+        authpair = None
 
         if service_url is None:
             # Figure out the path to the bitcoin.conf file
@@ -158,13 +159,22 @@ class BaseProxy(object):
                 conf['rpcport'] = int(conf.get('rpcport', service_port))
                 conf['rpchost'] = conf.get('rpcconnect', 'localhost')
 
-                if 'rpcpassword' not in conf:
-                    raise ValueError('The value of rpcpassword not specified in the configuration file: %s' % btc_conf_file)
+                service_url = ('%s://%s:%d' %
+                    ('http', conf['rpchost'], conf['rpcport']))
 
-                service_url = ('%s://%s:%s@%s:%d' %
-                    ('http',
-                     conf['rpcuser'], conf['rpcpassword'],
-                     conf['rpchost'], conf['rpcport']))
+                cookie_dir = os.path.dirname(btc_conf_file)
+                if bitcoin.params.NAME != "mainnet":
+                    cookie_dir = os.path.join(cookie_dir, bitcoin.params.NAME)
+                cookie_file = os.path.join(cookie_dir, ".cookie")
+                try:
+                    with open(cookie_file, 'r') as fd:
+                        authpair = fd.read()
+                except IOError as err:
+                    if 'rpcpassword' in conf:
+                        authpair = "%s:%s" % (conf['rpcuser'], conf['rpcpassword'])
+
+                    else:
+                        raise ValueError('Cookie file unusable (%s) and rpcpassword not specified in the configuration file: %r' % (err, btc_conf_file))
 
         self.__service_url = service_url
         self.__url = urlparse.urlparse(service_url)
@@ -177,14 +187,16 @@ class BaseProxy(object):
         else:
             port = self.__url.port
         self.__id_count = 0
-        authpair = "%s:%s" % (self.__url.username, self.__url.password)
-        authpair = authpair.encode('utf8')
-        self.__auth_header = b"Basic " + base64.b64encode(authpair)
+
+        if authpair is None:
+            self.__auth_header = None
+        else:
+            authpair = authpair.encode('utf8')
+            self.__auth_header = b"Basic " + base64.b64encode(authpair)
 
         self.__conn = httplib.HTTPConnection(self.__url.hostname, port=port,
                                              timeout=timeout)
 
-
     def _call(self, service_name, *args):
         self.__id_count += 1
 
@@ -192,11 +204,17 @@ class BaseProxy(object):
                                'method': service_name,
                                'params': args,
                                'id': self.__id_count})
-        self.__conn.request('POST', self.__url.path, postdata,
-                            {'Host': self.__url.hostname,
-                             'User-Agent': DEFAULT_USER_AGENT,
-                             'Authorization': self.__auth_header,
-                             'Content-type': 'application/json'})
+
+        headers = {
+            'Host': self.__url.hostname,
+            'User-Agent': DEFAULT_USER_AGENT,
+            'Content-type': 'application/json',
+        }
+
+        if self.__auth_header is not None:
+            headers['Authorization'] = self.__auth_header
+
+        self.__conn.request('POST', self.__url.path, postdata, headers)
 
         response = self._get_response()
         if response['error'] is not None:
@@ -207,15 +225,19 @@ class BaseProxy(object):
         else:
             return response['result']
 
-
     def _batch(self, rpc_call_list):
         postdata = json.dumps(list(rpc_call_list))
-        self.__conn.request('POST', self.__url.path, postdata,
-                            {'Host': self.__url.hostname,
-                             'User-Agent': DEFAULT_USER_AGENT,
-                             'Authorization': self.__auth_header,
-                             'Content-type': 'application/json'})
 
+        headers = {
+            'Host': self.__url.hostname,
+            'User-Agent': DEFAULT_USER_AGENT,
+            'Content-type': 'application/json',
+        }
+
+        if self.__auth_header is not None:
+            headers['Authorization'] = self.__auth_header
+
+        self.__conn.request('POST', self.__url.path, postdata, headers)
         return self._get_response()
 
     def _get_response(self):
@@ -227,6 +249,10 @@ class BaseProxy(object):
         return json.loads(http_response.read().decode('utf8'),
                           parse_float=decimal.Decimal)
 
+    def close(self):
+        if self.__conn is not None:
+            self.__conn.close()
+
     def __del__(self):
         if self.__conn is not None:
             self.__conn.close()
@@ -271,7 +297,7 @@ class Proxy(BaseProxy):
     Unlike ``RawProxy``, data is passed as ``bitcoin.core`` objects or packed
     bytes, rather than JSON or hex strings. Not all methods are implemented
     yet; you can use ``call`` to access missing ones in a forward-compatible
-    way. Assumes Bitcoin Core version >= v0.13.0; older versions mostly work,
+    way. Assumes Bitcoin Core version >= v0.15.0; older versions mostly work,
     but there are a few incompatibilities.
     """
 
@@ -350,7 +376,7 @@ class Proxy(BaseProxy):
         r = self._call('getaccountaddress', account)
         return CBitcoinAddress(r)
 
-    def getbalance(self, account='*', minconf=1):
+    def getbalance(self, account='*', minconf=1, include_watchonly=False):
         """Get the balance
 
         account - The selected account. Defaults to "*" for entire wallet. It
@@ -358,8 +384,11 @@ class Proxy(BaseProxy):
 
         minconf - Only include transactions confirmed at least this many times.
         (default=1)
+
+        include_watchonly - Also include balance in watch-only addresses (see 'importaddress')
+        (default=False)
         """
-        r = self._call('getbalance', account, minconf)
+        r = self._call('getbalance', account, minconf, include_watchonly)
         return int(r*COIN)
 
     def getbestblockhash(self):
@@ -583,7 +612,13 @@ class Proxy(BaseProxy):
             del unspent['txid']
             del unspent['vout']
 
-            unspent['address'] = CBitcoinAddress(unspent['address'])
+            # address isn't always available as Bitcoin Core allows scripts w/o
+            # an address type to be imported into the wallet, e.g. non-p2sh
+            # segwit
+            try:
+                unspent['address'] = CBitcoinAddress(unspent['address'])
+            except KeyError:
+                pass
             unspent['scriptPubKey'] = CScript(unhexlify(unspent['scriptPubKey']))
             unspent['amount'] = int(unspent['amount'] * COIN)
             r2.append(unspent)
@@ -609,14 +644,17 @@ class Proxy(BaseProxy):
         return lx(r)
 
     def sendmany(self, fromaccount, payments, minconf=1, comment='', subtractfeefromamount=[]):
-        """Sent amount to a given address"""
+        """Send amount to given addresses.
+
+        payments - dict with {address: amount}
+        """
         json_payments = {str(addr):float(amount)/COIN
                          for addr, amount in payments.items()}
         r = self._call('sendmany', fromaccount, json_payments, minconf, comment, subtractfeefromamount)
         return lx(r)
 
     def sendtoaddress(self, addr, amount, comment='', commentto='', subtractfeefromamount=False):
-        """Sent amount to a given address"""
+        """Send amount to a given address"""
         addr = str(addr)
         amount = float(amount)/COIN
         r = self._call('sendtoaddress', addr, amount, comment, commentto, subtractfeefromamount)
@@ -654,6 +692,17 @@ class Proxy(BaseProxy):
             r['pubkey'] = unhexlify(r['pubkey'])
         return r
 
+    def unlockwallet(self, password, timeout=60):
+        """Stores the wallet decryption key in memory for 'timeout' seconds.
+
+        password - The wallet passphrase.
+
+        timeout - The time to keep the decryption key in seconds.
+        (default=60)
+        """
+        r = self._call('walletpassphrase', password, timeout)
+        return r
+
     def _addnode(self, node, arg):
         r = self._call('addnode', node, arg)
         return r
